(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = global || self, factory(global['react-async-hook'] = {}, global.React));
}(this, function (exports, react) { 'use strict';

  const InitialAsyncState = {
      loading: true,
      result: undefined,
      error: undefined,
  };
  const defaultSetLoading = _asyncState => InitialAsyncState;
  const defaultSetResult = (result, _asyncState) => ({
      loading: false,
      result: result,
      error: undefined,
  });
  const defaultSetError = (error, _asyncState) => ({
      loading: false,
      result: undefined,
      error: error,
  });
  const DefaultOptions = {
      initialState: InitialAsyncState,
      executeOnMount: true,
      executeOnUpdate: true,
      setLoading: defaultSetLoading,
      setResult: defaultSetResult,
      setError: defaultSetError,
  };
  const normalizeOptions = (options) => ({
      ...DefaultOptions,
      ...options,
  });
  const useAsyncState = (options) => {
      const [value, setValue] = react.useState(InitialAsyncState);
      return {
          value,
          setLoading: () => setValue(options.setLoading(value)),
          setResult: result => setValue(options.setResult(result, value)),
          setError: error => setValue(options.setError(error, value)),
      };
  };
  const useIsMounted = () => {
      const ref = react.useRef(false);
      react.useEffect(() => {
          ref.current = true;
          return () => {
              ref.current = false;
          };
      }, []);
      return () => ref.current;
  };
  const useCurrentPromise = () => {
      const ref = react.useRef(null);
      return {
          set: promise => (ref.current = promise),
          get: () => ref.current,
          is: promise => ref.current === promise,
      };
  };
  const useAsync = (asyncFunction, params, options) => {
      const normalizedOptions = normalizeOptions(options);
      const AsyncState = useAsyncState(normalizedOptions);
      const isMounted = useIsMounted();
      const CurrentPromise = useCurrentPromise();
      // We only want to handle the promise result/error
      // if it is the last operation and the comp is still mounted
      const shouldHandlePromise = (p) => isMounted() && CurrentPromise.is(p);
      const executeAsyncOperation = () => {
          const promise = asyncFunction(...params);
          CurrentPromise.set(promise);
          AsyncState.setLoading();
          promise.then(result => {
              if (shouldHandlePromise(promise)) {
                  AsyncState.setResult(result);
              }
          }, error => {
              if (shouldHandlePromise(promise)) {
                  AsyncState.setError(error);
              }
          });
          return promise;
      };
      // Keep this outside useEffect, because inside isMounted()
      // will be true as the component is already mounted when it's run
      const isMounting = !isMounted();
      react.useEffect(() => {
          if (isMounting) {
              normalizedOptions.executeOnMount && executeAsyncOperation();
          }
          else {
              normalizedOptions.executeOnUpdate && executeAsyncOperation();
          }
      }, params);
      return {
          ...AsyncState.value,
          execute: executeAsyncOperation,
          currentPromise: CurrentPromise.get(),
      };
  };
  const useAsyncAbortable = (asyncFunction, params, options) => {
      const abortControllerRef = react.useRef();
      // Wrap the original async function and enhance it with abortion login
      const asyncFunctionWrapper = async (...p) => {
          // Cancel previous async call
          if (abortControllerRef.current) {
              abortControllerRef.current.abort();
          }
          // Create/store new abort controller for next async call
          const abortController = new AbortController();
          abortControllerRef.current = abortController;
          try {
              // @ts-ignore // TODO how to type this?
              return await asyncFunction(abortController.signal, ...p);
          }
          finally {
              // Unset abortController ref if response is already there,
              // as it's not needed anymore to try to abort it (would it be no-op?)
              if (abortControllerRef.current === abortController) {
                  abortControllerRef.current = undefined;
              }
          }
      };
      return useAsync(asyncFunctionWrapper, params, options);
  };

  exports.useAsync = useAsync;
  exports.useAsyncAbortable = useAsyncAbortable;

}));
//# sourceMappingURL=react-async-hook.umd.development.js.map
